/*
 * Node to be run on the host machine
 * Subscribes to joystick inputs, publishes instructions for create2
 * Borrowed heavily from: http://wiki.ros.org/ps3joy/Tutorials/WritingTeleopNode
 *
 */

#include <ros/ros.h>
#include <sensor_msgs/Joy.h>
#include <irobotcreate2/instructions.h>
#include <math.h> //for cubic drive
#define DATA_LENGTH 4

class Create2Teleop {
public:
	Create2Teleop();
private:
	void joyCallback(const sensor_msgs::Joy::ConstPtr& joy);
	void timerCallback(const ros::TimerEvent& event);	
	ros::NodeHandle nh;
	ros::Publisher instruction_pub;
	ros::Subscriber joy_sub;
	ros::Timer joy_timer;
	irobotcreate2::instructions toMove; //message to publish 
	float instructions[4];
};

Create2Teleop::Create2Teleop() {
	//initialize publisher and subscriber
	instruction_pub = nh.advertise<irobotcreate2::instructions>("instructions", 1);
	joy_sub = nh.subscribe<sensor_msgs::Joy>("joy", 1, &Create2Teleop::joyCallback, this);
}	

void Create2Teleop::joyCallback(const sensor_msgs::Joy::ConstPtr& joy) {
//	int i = 0;
//	for(i = 0; i < 4; i++)
//		ROS_INFO("bleh axis %d is at position %f", i, joy->axes[i]);
	toMove.left_stick_left_right = pow(joy->axes[0],3);
    toMove.left_stick_up_down = pow(joy->axes[1],3);
	toMove.right_stick_left_right = pow(joy->axes[2],3);
	toMove.right_stick_up_down = pow(joy->axes[3],3);	
	instruction_pub.publish(toMove);

	//create timer if timeouts
	if(joy_timer) {
		joy_timer.stop();
	}

	joy_timer = nh.createTimer(ros::Duration(10), &Create2Teleop::timerCallback, this, true);
}


//publish 0's if nothing received after a certain time.
void Create2Teleop::timerCallback(const ros::TimerEvent& event) {
	toMove.left_stick_left_right = 0;
    toMove.left_stick_up_down = 0;
	toMove.right_stick_left_right = 0;
	toMove.right_stick_up_down = 0;
	instruction_pub.publish(toMove);
}

int main(int argc, char** argv) {
	ros::init(argc, argv, "teleop_create2");
	Create2Teleop teleop_create2;
	ros::spin();

	return 0;
}
